@@ -38,11 +38,11 @@ from .entities.opencti_stix_sighting_relationship import StixSightingRelationshi
 from .entities.opencti_threat_actor import ThreatActor
 from .entities.opencti_tool import Tool
 from .entities.opencti_vulnerability import Vulnerability
-from .utils.constants import StixCyberObservableTypes
+from .utils.constants import StixCyberObservableTypes, StixMetaTypes
 from .utils.opencti_stix2 import OpenCTIStix2
 from .utils.opencti_stix2_splitter import OpenCTIStix2Splitter
 from .utils.opencti_stix2_update import OpenCTIStix2Update
-from .utils.opencti_stix2_utils import OpenCTIStix2Utils, SimpleObservable, StixIncident
+from .utils.opencti_stix2_utils import OpenCTIStix2Utils, SimpleObservable
 
 __all__ = [
     "OpenCTIApiClient",
@@ -84,6 +84,6 @@ __all__ = [
     "OpenCTIStix2Update",
     "OpenCTIStix2Utils",
     "StixCyberObservableTypes",
+    "StixMetaTypes",
     "SimpleObservable",
-    "StixIncident",
 ]
