@@ -1,12 +1,13 @@
 # coding: utf-8
-
 import datetime
 import io
 import json
 import logging
+import base64
 from typing import Union
 
 import magic
+from pythonjsonlogger import jsonlogger
 import requests
 import urllib3
 
@@ -30,6 +31,8 @@ from pycti.entities.opencti_note import Note
 from pycti.entities.opencti_observed_data import ObservedData
 from pycti.entities.opencti_opinion import Opinion
 from pycti.entities.opencti_report import Report
+from pycti.entities.opencti_stix import Stix
+from pycti.entities.opencti_stix_core_object import StixCoreObject
 from pycti.entities.opencti_stix_core_relationship import StixCoreRelationship
 from pycti.entities.opencti_stix_cyber_observable import StixCyberObservable
 from pycti.entities.opencti_stix_cyber_observable_relationship import (
@@ -48,6 +51,19 @@ from pycti.utils.opencti_stix2 import OpenCTIStix2
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 
+class CustomJsonFormatter(jsonlogger.JsonFormatter):
+    def add_fields(self, log_record, record, message_dict):
+        super(CustomJsonFormatter, self).add_fields(log_record, record, message_dict)
+        if not log_record.get("timestamp"):
+            # This doesn't use record.created, so it is slightly off
+            now = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
+            log_record["timestamp"] = now
+        if log_record.get("level"):
+            log_record["level"] = log_record["level"].upper()
+        else:
+            log_record["level"] = record.levelname
+
+
 class File:
     def __init__(self, name, data, mime="text/plain"):
         self.name = name
@@ -74,9 +90,19 @@ class OpenCTIApiClient:
             "https: "http://my_proxy:8080"
         }
         ```
+    :param json_logging: format the logs as json if set to True
+    :type json_logging: bool, optional
     """
 
-    def __init__(self, url, token, log_level="info", ssl_verify=False, proxies={}):
+    def __init__(
+        self,
+        url,
+        token,
+        log_level="info",
+        ssl_verify=False,
+        proxies=None,
+        json_logging=False,
+    ):
         """Constructor method"""
 
         # Check configuration
@@ -92,7 +118,17 @@ class OpenCTIApiClient:
         numeric_level = getattr(logging, self.log_level.upper(), None)
         if not isinstance(numeric_level, int):
             raise ValueError("Invalid log level: " + self.log_level)
-        logging.basicConfig(level=numeric_level)
+
+        if json_logging:
+            log_handler = logging.StreamHandler()
+            log_handler.setLevel(self.log_level.upper())
+            formatter = CustomJsonFormatter(
+                "%(timestamp)s %(level)s %(name)s %(message)s"
+            )
+            log_handler.setFormatter(formatter)
+            logging.basicConfig(handlers=[log_handler], level=numeric_level, force=True)
+        else:
+            logging.basicConfig(level=numeric_level)
 
         # Define API
         self.api_token = token
@@ -108,12 +144,14 @@ class OpenCTIApiClient:
         # Define the entities
         self.label = Label(self)
         self.marking_definition = MarkingDefinition(self)
-        self.external_reference = ExternalReference(self)
+        self.external_reference = ExternalReference(self, File)
         self.kill_chain_phase = KillChainPhase(self)
         self.opencti_stix_object_or_stix_relationship = StixObjectOrStixRelationship(
             self
         )
+        self.stix = Stix(self)
         self.stix_domain_object = StixDomainObject(self, File)
+        self.stix_core_object = StixCoreObject(self, File)
         self.stix_cyber_observable = StixCyberObservable(self, File)
         self.stix_core_relationship = StixCoreRelationship(self)
         self.stix_sighting_relationship = StixSightingRelationship(self)
@@ -282,7 +320,7 @@ class OpenCTIApiClient:
             logging.info(r.text)
             raise ValueError(r.text)
 
-    def fetch_opencti_file(self, fetch_uri, binary=False):
+    def fetch_opencti_file(self, fetch_uri, binary=False, serialize=False):
         """get file from the OpenCTI API
 
         :param fetch_uri: download URI to use
@@ -295,7 +333,11 @@ class OpenCTIApiClient:
 
         r = self.session.get(fetch_uri, headers=self.request_headers)
         if binary:
+            if serialize:
+                return base64.b64encode(r.content).decode("utf-8")
             return r.content
+        if serialize:
+            return base64.b64encode(r.text).decode("utf-8")
         return r.text
 
     def log(self, level, message):
@@ -544,3 +586,43 @@ class OpenCTIApiClient:
                 "[upload] Missing parameters: file_name or data",
             )
             return None
+
+    def upload_pending_file(self, **kwargs):
+        """upload a file to OpenCTI API
+
+        :param `**kwargs`: arguments for file upload (required: `file_name` and `data`)
+        :return: returns the query respons for the file upload
+        :rtype: dict
+        """
+
+        file_name = kwargs.get("file_name", None)
+        data = kwargs.get("data", None)
+        mime_type = kwargs.get("mime_type", "text/plain")
+        entity_id = kwargs.get("entity_id", None)
+
+        if file_name is not None:
+            self.log("info", "Uploading a file.")
+            query = """
+                    mutation UploadPending($file: Upload!, $entityId: String) {
+                        uploadPending(file: $file, entityId: $entityId) {
+                            id
+                            name
+                        }
+                    }
+                 """
+            if data is None:
+                data = open(file_name, "rb")
+                if file_name.endswith(".json"):
+                    mime_type = "application/json"
+                else:
+                    mime_type = magic.from_file(file_name, mime=True)
+            return self.query(
+                query,
+                {"file": (File(file_name, data, mime_type)), "entityId": entity_id},
+            )
+        else:
+            self.log(
+                "error",
+                "[upload] Missing parameters: file_name or data",
+            )
+            return None
