@@ -4,6 +4,7 @@ import json
 import logging
 import os
 import ssl
+import sys
 import threading
 import time
 import uuid
@@ -17,10 +18,13 @@ from pycti.api.opencti_api_client import OpenCTIApiClient
 from pycti.connector.opencti_connector import OpenCTIConnector
 from pycti.utils.opencti_stix2_splitter import OpenCTIStix2Splitter
 
+TRUTHY: List[str] = ["yes", "true", "True"]
+FALSY: List[str] = ["no", "false", "False"]
+
 
 def get_config_variable(
     env_var: str,
-    yaml_path: list,
+    yaml_path: List,
     config: Dict = {},
     isNumber: Optional[bool] = False,
     default=None,
@@ -43,14 +47,14 @@ def get_config_variable(
     else:
         return default
 
-    if result == "yes" or result == "true" or result == "True":
+    if result in TRUTHY:
         return True
-    elif result == "no" or result == "false" or result == "False":
+    if result in FALSY:
         return False
-    elif isNumber:
+    if isNumber:
         return int(result)
-    else:
-        return result
+
+    return result
 
 
 def create_ssl_context() -> ssl.SSLContext:
@@ -83,12 +87,12 @@ class ListenQueue(threading.Thread):
     :param helper: instance of a `OpenCTIConnectorHelper` class
     :type helper: OpenCTIConnectorHelper
     :param config: dict containing client config
-    :type config: dict
+    :type config: Dict
     :param callback: callback function to process queue
     :type callback: callable
     """
 
-    def __init__(self, helper, config: dict, callback):
+    def __init__(self, helper, config: Dict, callback) -> None:
         threading.Thread.__init__(self)
         self.pika_credentials = None
         self.pika_parameters = None
@@ -102,9 +106,11 @@ class ListenQueue(threading.Thread):
         self.user = config["connection"]["user"]
         self.password = config["connection"]["pass"]
         self.queue_name = config["listen"]
+        self.exit_event = threading.Event()
+        self.thread = None
 
     # noinspection PyUnusedLocal
-    def _process_message(self, channel, method, properties, body):
+    def _process_message(self, channel, method, properties, body) -> None:
         """process a message from the rabbit queue
 
         :param channel: channel instance
@@ -118,18 +124,30 @@ class ListenQueue(threading.Thread):
         """
 
         json_data = json.loads(body)
-        thread = threading.Thread(target=self._data_handler, args=[json_data])
-        thread.start()
-        while thread.is_alive():  # Loop while the thread is processing
-            self.pika_connection.sleep(1.0)
-        logging.info(
-            "Message (delivery_tag="
-            + str(method.delivery_tag)
-            + ") processed, thread terminated"
-        )
         channel.basic_ack(delivery_tag=method.delivery_tag)
+        self.thread = threading.Thread(target=self._data_handler, args=[json_data])
+        self.thread.start()
+        five_minutes = 60 * 5
+        time_wait = 0
+        while self.thread.is_alive():  # Loop while the thread is processing
+            if (
+                self.helper.work_id is not None and time_wait > five_minutes
+            ):  # Ping every 5 minutes
+                self.helper.api.work.ping(self.helper.work_id)
+                time_wait = 0
+            else:
+                time_wait += 1
+            time.sleep(1)
 
-    def _data_handler(self, json_data):
+        logging.info(
+            "%s",
+            (
+                f"Message (delivery_tag={method.delivery_tag}) processed"
+                ", thread terminated"
+            ),
+        )
+
+    def _data_handler(self, json_data) -> None:
         # Set the API headers
         work_id = json_data["internal"]["work_id"]
         applicant_id = json_data["internal"]["applicant_id"]
@@ -144,15 +162,15 @@ class ListenQueue(threading.Thread):
             )
             message = self.callback(json_data["event"])
             self.helper.api.work.to_processed(work_id, message)
-        except Exception as e:
+        except Exception as e:  # pylint: disable=broad-except
             logging.exception("Error in message processing, reporting error to API")
             try:
                 self.helper.api.work.to_processed(work_id, str(e), True)
-            except:
+            except:  # pylint: disable=bare-except
                 logging.error("Failing reporting the processing")
 
-    def run(self):
-        while True:
+    def run(self) -> None:
+        while not self.exit_event.is_set():
             try:
                 # Connect the broker
                 self.pika_credentials = pika.PlainCredentials(self.user, self.password)
@@ -167,59 +185,75 @@ class ListenQueue(threading.Thread):
                 )
                 self.pika_connection = pika.BlockingConnection(self.pika_parameters)
                 self.channel = self.pika_connection.channel()
+                assert self.channel is not None
                 self.channel.basic_consume(
                     queue=self.queue_name, on_message_callback=self._process_message
                 )
                 self.channel.start_consuming()
             except (KeyboardInterrupt, SystemExit):
                 self.helper.log_info("Connector stop")
-                exit(0)
-            except Exception as e:
+                sys.exit(0)
+            except Exception as e:  # pylint: disable=broad-except
                 self.helper.log_error(str(e))
                 time.sleep(10)
 
+    def stop(self):
+        self.exit_event.set()
+        if self.thread:
+            self.thread.join()
+
 
 class PingAlive(threading.Thread):
-    def __init__(self, connector_id, api, get_state, set_state):
+    def __init__(self, connector_id, api, get_state, set_state) -> None:
         threading.Thread.__init__(self)
         self.connector_id = connector_id
         self.in_error = False
         self.api = api
         self.get_state = get_state
         self.set_state = set_state
+        self.exit_event = threading.Event()
 
-    def ping(self):
-        while True:
+    def ping(self) -> None:
+        while not self.exit_event.is_set():
             try:
                 initial_state = self.get_state()
                 result = self.api.connector.ping(self.connector_id, initial_state)
                 remote_state = (
                     json.loads(result["connector_state"])
-                    if len(result["connector_state"]) > 0
+                    if result["connector_state"] is not None
+                    and len(result["connector_state"]) > 0
                     else None
                 )
                 if initial_state != remote_state:
                     self.set_state(result["connector_state"])
                     logging.info(
-                        'Connector state has been remotely reset to: "'
-                        + self.get_state()
-                        + '"'
+                        "%s",
+                        (
+                            "Connector state has been remotely reset to: "
+                            f'"{self.get_state()}"'
+                        ),
                     )
                 if self.in_error:
                     self.in_error = False
                     logging.error("API Ping back to normal")
-            except Exception:
+            except Exception:  # pylint: disable=broad-except
                 self.in_error = True
                 logging.error("Error pinging the API")
-            time.sleep(40)
+            self.exit_event.wait(40)
 
-    def run(self):
+    def run(self) -> None:
         logging.info("Starting ping alive thread")
         self.ping()
 
+    def stop(self) -> None:
+        logging.info("Preparing for clean shutdown")
+        self.exit_event.set()
+
 
 class ListenStream(threading.Thread):
-    def __init__(self, helper, callback, url, token, verify_ssl, start_timestamp):
+    def __init__(
+        self, helper, callback, url, token, verify_ssl, start_timestamp, live_stream_id
+    ) -> None:
         threading.Thread.__init__(self)
         self.helper = helper
         self.callback = callback
@@ -227,12 +261,14 @@ class ListenStream(threading.Thread):
         self.token = token
         self.verify_ssl = verify_ssl
         self.start_timestamp = start_timestamp
+        self.live_stream_id = live_stream_id
+        self.exit = False
 
-    def run(self):
+    def run(self) -> None:  # pylint: disable=too-many-branches
         current_state = self.helper.get_state()
         if current_state is None:
             current_state = {
-                "connectorLastEventId": str(self.start_timestamp) + "-0"
+                "connectorLastEventId": f"{self.start_timestamp}-0"
                 if self.start_timestamp is not None and len(self.start_timestamp) > 0
                 else "-"
             }
@@ -241,15 +277,19 @@ class ListenStream(threading.Thread):
         # If URL and token are provided, likely consuming a remote stream
         if self.url is not None and self.token is not None:
             # If a live stream ID, appending the URL
-            live_stream_uri = (
-                ("/" + self.helper.connect_live_stream_id)
-                if self.helper.connect_live_stream_id is not None
-                else ""
-            )
+            if self.live_stream_id is not None:
+                live_stream_uri = f"/{self.live_stream_id}"
+            elif self.helper.connect_live_stream_id is not None:
+                live_stream_uri = f"/{self.helper.connect_live_stream_id}"
+            else:
+                live_stream_uri = ""
             # Live stream "from" should be empty if start from the beginning
-            if self.helper.connect_live_stream_id is not None:
+            if (
+                self.live_stream_id is not None
+                or self.helper.connect_live_stream_id is not None
+            ):
                 live_stream_from = (
-                    ("?from=" + current_state["connectorLastEventId"])
+                    f"?from={current_state['connectorLastEventId']}"
                     if current_state["connectorLastEventId"] != "-"
                     else ""
                 )
@@ -260,31 +300,31 @@ class ListenStream(threading.Thread):
                     if current_state["connectorLastEventId"] != "-"
                     else "0"
                 )
-            live_stream_url = self.url + "/stream" + live_stream_uri + live_stream_from
+            live_stream_url = f"{self.url}/stream{live_stream_uri}{live_stream_from}"
             opencti_ssl_verify = (
                 self.verify_ssl if self.verify_ssl is not None else True
             )
             logging.info(
-                "Starting listening stream events (URL: "
-                + live_stream_url
-                + ", SSL verify: "
-                + str(opencti_ssl_verify)
-                + ")"
+                "%s",
+                (
+                    "Starting listening stream events (URL: "
+                    f"{live_stream_url}, SSL verify: {opencti_ssl_verify})"
+                ),
             )
             messages = SSEClient(
                 live_stream_url,
-                headers={"Authorization": "Bearer " + self.token},
+                headers={"authorization": "Bearer " + self.token},
                 verify=opencti_ssl_verify,
             )
         else:
             live_stream_uri = (
-                ("/" + self.helper.connect_live_stream_id)
+                f"/{self.helper.connect_live_stream_id}"
                 if self.helper.connect_live_stream_id is not None
                 else ""
             )
             if self.helper.connect_live_stream_id is not None:
                 live_stream_from = (
-                    ("?from=" + current_state["connectorLastEventId"])
+                    f"?from={current_state['connectorLastEventId']}"
                     if current_state["connectorLastEventId"] != "-"
                     else ""
                 )
@@ -296,25 +336,27 @@ class ListenStream(threading.Thread):
                     else "0"
                 )
             live_stream_url = (
-                self.helper.opencti_url + "/stream" + live_stream_uri + live_stream_from
+                f"{self.helper.opencti_url}/stream{live_stream_uri}{live_stream_from}"
             )
             logging.info(
-                "Starting listening stream events (URL: "
-                + live_stream_url
-                + ", SSL verify: "
-                + str(self.helper.opencti_ssl_verify)
-                + ")"
+                "%s",
+                (
+                    f"Starting listening stream events (URL: {live_stream_url}"
+                    f", SSL verify: {self.helper.opencti_ssl_verify})"
+                ),
             )
             messages = SSEClient(
                 live_stream_url,
-                headers={"Authorization": "Bearer " + self.helper.opencti_token},
+                headers={"authorization": "Bearer " + self.helper.opencti_token},
                 verify=self.helper.opencti_ssl_verify,
             )
-
+        # Iter on stream messages
         for msg in messages:
+            if self.exit:
+                break
             if msg.event == "heartbeat" or msg.event == "connected":
                 continue
-            elif msg.event == "sync":
+            if msg.event == "sync":
                 if msg.id is not None:
                     state = self.helper.get_state()
                     state["connectorLastEventId"] = str(msg.id)
@@ -326,15 +368,18 @@ class ListenStream(threading.Thread):
                     state["connectorLastEventId"] = str(msg.id)
                     self.helper.set_state(state)
 
+    def stop(self):
+        self.exit = True
 
-class OpenCTIConnectorHelper:
+
+class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
     """Python API for OpenCTI connector
 
-    :param config: Dict standard config
-    :type config: dict
+    :param config: dict standard config
+    :type config: Dict
     """
 
-    def __init__(self, config: dict):
+    def __init__(self, config: Dict) -> None:
         # Load API config
         self.opencti_url = get_config_variable(
             "OPENCTI_URL", ["opencti", "url"], config
@@ -345,6 +390,9 @@ class OpenCTIConnectorHelper:
         self.opencti_ssl_verify = get_config_variable(
             "OPENCTI_SSL_VERIFY", ["opencti", "ssl_verify"], config, False, True
         )
+        self.opencti_json_logging = get_config_variable(
+            "OPENCTI_JSON_LOGGING", ["opencti", "json_logging"], config
+        )
         # Load connector config
         self.connect_id = get_config_variable(
             "CONNECTOR_ID", ["connector", "id"], config
@@ -384,18 +432,35 @@ class OpenCTIConnectorHelper:
         self.log_level = get_config_variable(
             "CONNECTOR_LOG_LEVEL", ["connector", "log_level"], config
         )
+        self.connect_run_and_terminate = get_config_variable(
+            "CONNECTOR_RUN_AND_TERMINATE",
+            ["connector", "run_and_terminate"],
+            config,
+            False,
+            False,
+        )
+        self.connect_validate_before_import = get_config_variable(
+            "CONNECTOR_VALIDATE_BEFORE_IMPORT",
+            ["connector", "validate_before_import"],
+            config,
+            False,
+            False,
+        )
 
         # Configure logger
         numeric_level = getattr(
             logging, self.log_level.upper() if self.log_level else "INFO", None
         )
         if not isinstance(numeric_level, int):
-            raise ValueError("Invalid log level: " + self.log_level)
+            raise ValueError(f"Invalid log level: {self.log_level}")
         logging.basicConfig(level=numeric_level)
 
         # Initialize configuration
         self.api = OpenCTIApiClient(
-            self.opencti_url, self.opencti_token, self.log_level
+            self.opencti_url,
+            self.opencti_token,
+            self.log_level,
+            json_logging=self.opencti_json_logging,
         )
         # Register the connector in OpenCTI
         self.connector = OpenCTIConnector(
@@ -407,7 +472,7 @@ class OpenCTIConnectorHelper:
             self.connect_only_contextual,
         )
         connector_configuration = self.api.connector.register(self.connector)
-        logging.info("Connector registered with ID:" + self.connect_id)
+        logging.info("%s", f"Connector registered with ID: {self.connect_id}")
         self.connector_id = connector_configuration["id"]
         self.work_id = None
         self.applicant_id = connector_configuration["connector_user"]["id"]
@@ -415,27 +480,45 @@ class OpenCTIConnectorHelper:
         self.config = connector_configuration["config"]
 
         # Start ping thread
-        self.ping = PingAlive(
-            self.connector.id, self.api, self.get_state, self.set_state
-        )
-        self.ping.start()
+        if not self.connect_run_and_terminate:
+            self.ping = PingAlive(
+                self.connector.id, self.api, self.get_state, self.set_state
+            )
+            self.ping.start()
 
-    def get_name(self):
+        # self.listen_stream = None
+        self.listen_queue = None
+
+    def stop(self) -> None:
+        if self.listen_queue:
+            self.listen_queue.stop()
+        # if self.listen_stream:
+        #     self.listen_stream.stop()
+        self.ping.stop()
+        self.api.connector.unregister(self.connector_id)
+
+    def get_name(self) -> Optional[Union[bool, int, str]]:
         return self.connect_name
 
-    def get_only_contextual(self):
+    def get_only_contextual(self) -> Optional[Union[bool, int, str]]:
         return self.connect_only_contextual
 
+    def get_run_and_terminate(self) -> Optional[Union[bool, int, str]]:
+        return self.connect_run_and_terminate
+
+    def get_validate_before_import(self) -> Optional[Union[bool, int, str]]:
+        return self.connect_validate_before_import
+
     def set_state(self, state) -> None:
         """sets the connector state
 
         :param state: state object
-        :type state: dict
+        :type state: Dict
         """
 
         self.connector_state = json.dumps(state)
 
-    def get_state(self):
+    def get_state(self) -> Optional[Dict]:
         """get the connector state
 
         :return: returns the current state of the connector if there is any
@@ -445,12 +528,27 @@ class OpenCTIConnectorHelper:
         try:
             if self.connector_state:
                 state = json.loads(self.connector_state)
-                if isinstance(state, dict) and state:
+                if isinstance(state, Dict) and state:
                     return state
-        except:
+        except:  # pylint: disable=bare-except
             pass
         return None
 
+    def force_ping(self):
+        try:
+            initial_state = self.get_state()
+            result = self.api.connector.ping(self.connector_id, initial_state)
+            remote_state = (
+                json.loads(result["connector_state"])
+                if result["connector_state"] is not None
+                and len(result["connector_state"]) > 0
+                else None
+            )
+            if initial_state != remote_state:
+                self.api.connector.ping(self.connector_id, initial_state)
+        except Exception:  # pylint: disable=broad-except
+            logging.error("Error pinging the API")
+
     def listen(self, message_callback: Callable[[Dict], str]) -> None:
         """listen for messages and register callback function
 
@@ -458,8 +556,8 @@ class OpenCTIConnectorHelper:
         :type message_callback: Callable[[Dict], str]
         """
 
-        listen_queue = ListenQueue(self, self.config, message_callback)
-        listen_queue.start()
+        self.listen_queue = ListenQueue(self, self.config, message_callback)
+        self.listen_queue.start()
 
     def listen_stream(
         self,
@@ -468,36 +566,44 @@ class OpenCTIConnectorHelper:
         token=None,
         verify_ssl=None,
         start_timestamp=None,
-    ) -> None:
+        live_stream_id=None,
+    ) -> ListenStream:
         """listen for messages and register callback function
 
         :param message_callback: callback function to process messages
         """
 
-        listen_stream = ListenStream(
-            self, message_callback, url, token, verify_ssl, start_timestamp
+        self.listen_stream = ListenStream(
+            self,
+            message_callback,
+            url,
+            token,
+            verify_ssl,
+            start_timestamp,
+            live_stream_id,
         )
-        listen_stream.start()
+        self.listen_stream.start()
+        return self.listen_stream
 
-    def get_opencti_url(self):
+    def get_opencti_url(self) -> Optional[Union[bool, int, str]]:
         return self.opencti_url
 
-    def get_opencti_token(self):
+    def get_opencti_token(self) -> Optional[Union[bool, int, str]]:
         return self.opencti_token
 
-    def get_connector(self):
+    def get_connector(self) -> OpenCTIConnector:
         return self.connector
 
-    def log_error(self, msg):
+    def log_error(self, msg: str) -> None:
         logging.error(msg)
 
-    def log_info(self, msg):
+    def log_info(self, msg: str) -> None:
         logging.info(msg)
 
-    def log_debug(self, msg):
+    def log_debug(self, msg: str) -> None:
         logging.debug(msg)
 
-    def log_warning(self, msg):
+    def log_warning(self, msg: str) -> None:
         logging.warning(msg)
 
     def date_now(self) -> str:
@@ -530,11 +636,26 @@ class OpenCTIConnectorHelper:
         entities_types = kwargs.get("entities_types", None)
         update = kwargs.get("update", False)
         event_version = kwargs.get("event_version", None)
+        bypass_split = kwargs.get("bypass_split", False)
+        bypass_validation = kwargs.get("bypass_validation", False)
+        file_name = kwargs.get("file_name", work_id + ".json")
+        entity_id = kwargs.get("entity_id", None)
 
+        if self.connect_validate_before_import and not bypass_validation:
+            self.api.upload_pending_file(
+                file_name=file_name,
+                data=bundle,
+                mime_type="application/json",
+                entity_id=entity_id,
+            )
+            return []
         if entities_types is None:
             entities_types = []
-        stix2_splitter = OpenCTIStix2Splitter()
-        bundles = stix2_splitter.split_bundle(bundle, True, event_version)
+        if bypass_split:
+            bundles = [bundle]
+        else:
+            stix2_splitter = OpenCTIStix2Splitter()
+            bundles = stix2_splitter.split_bundle(bundle, True, event_version)
         if len(bundles) == 0:
             raise ValueError("Nothing to import")
         if work_id is not None:
@@ -619,7 +740,7 @@ class OpenCTIConnectorHelper:
             )
             logging.info("Bundle has been sent")
         except (UnroutableError, NackError) as e:
-            logging.error("Unable to send bundle, retry...", e)
+            logging.error("Unable to send bundle, retry...%s", e)
             self._send_bundle(channel, bundle, **kwargs)
 
     def split_stix2_bundle(self, bundle) -> list:
@@ -636,8 +757,8 @@ class OpenCTIConnectorHelper:
         self.cache_added = []
         try:
             bundle_data = json.loads(bundle)
-        except:
-            raise Exception("File data is not a valid JSON")
+        except Exception as e:
+            raise Exception("File data is not a valid JSON") from e
 
         # validation = validate_parsed_json(bundle_data)
         # if not validation.is_valid:
@@ -680,13 +801,13 @@ class OpenCTIConnectorHelper:
 
         return bundles
 
-    def stix2_get_embedded_objects(self, item) -> dict:
+    def stix2_get_embedded_objects(self, item) -> Dict:
         """gets created and marking refs for a stix2 item
 
         :param item: valid stix2 item
         :type item:
         :return: returns a dict of created_by of object_marking_refs
-        :rtype: dict
+        :rtype: Dict
         """
         # Marking definitions
         object_marking_refs = []
@@ -793,7 +914,7 @@ class OpenCTIConnectorHelper:
         return final_items
 
     @staticmethod
-    def stix2_create_bundle(items):
+    def stix2_create_bundle(items) -> Optional[str]:
         """create a stix2 bundle with items
 
         :param items: valid stix2 items
@@ -804,30 +925,29 @@ class OpenCTIConnectorHelper:
 
         bundle = {
             "type": "bundle",
-            "id": "bundle--" + str(uuid.uuid4()),
+            "id": f"bundle--{uuid.uuid4()}",
             "spec_version": "2.0",
             "objects": items,
         }
         return json.dumps(bundle)
 
     @staticmethod
-    def check_max_tlp(tlp, max_tlp) -> bool:
+    def check_max_tlp(tlp: str, max_tlp: str) -> bool:
         """check the allowed TLP levels for a TLP string
 
         :param tlp: string for TLP level to check
         :type tlp: str
         :param max_tlp: the highest allowed TLP level
         :type max_tlp: str
-        :return: list of allowed TLP levels
+        :return: TLP level in allowed TLPs
         :rtype: bool
         """
 
-        allowed_tlps = ["TLP:WHITE"]
-        if max_tlp == "TLP:RED":
-            allowed_tlps = ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER", "TLP:RED"]
-        elif max_tlp == "TLP:AMBER":
-            allowed_tlps = ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER"]
-        elif max_tlp == "TLP:GREEN":
-            allowed_tlps = ["TLP:WHITE", "TLP:GREEN"]
+        allowed_tlps: Dict[str, List[str]] = {
+            "TLP:RED": ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER", "TLP:RED"],
+            "TLP:AMBER": ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER"],
+            "TLP:GREEN": ["TLP:WHITE", "TLP:GREEN"],
+            "TLP:WHITE": ["TLP:WHITE"],
+        }
 
-        return tlp in allowed_tlps
+        return tlp in allowed_tlps[max_tlp]
