@@ -1,24 +1,41 @@
 # coding: utf-8
 
 import json
+import os
+import magic
 
 
 class ExternalReference:
-    def __init__(self, opencti):
+    def __init__(self, opencti, file):
         self.opencti = opencti
+        self.file = file
         self.properties = """
             id
+            standard_id
             entity_type
-            stix_id_key
+            parent_types
+            created_at
+            updated_at
+            created
+            modified
             source_name
             description
             url
             hash
             external_id
-            created
-            modified
-            created_at
-            updated_at
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
     """
@@ -36,6 +53,7 @@ class ExternalReference:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
@@ -52,7 +70,7 @@ class ExternalReference:
                     edges {
                         node {
                             """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -129,15 +147,16 @@ class ExternalReference:
         :return External Reference object
     """
 
-    def create_raw(self, **kwargs):
+    def create(self, **kwargs):
+        stix_id = kwargs.get("stix_id", None)
+        created = kwargs.get("created", None)
+        modified = kwargs.get("modified", None)
         source_name = kwargs.get("source_name", None)
         url = kwargs.get("url", None)
         external_id = kwargs.get("external_id", None)
         description = kwargs.get("description", None)
-        id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
-        created = kwargs.get("created", None)
-        modified = kwargs.get("modified", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        update = kwargs.get("update", False)
 
         if source_name is not None and url is not None:
             self.opencti.log(
@@ -158,14 +177,15 @@ class ExternalReference:
                 query,
                 {
                     "input": {
+                        "stix_id": stix_id,
+                        "created": created,
+                        "modified": modified,
                         "source_name": source_name,
                         "external_id": external_id,
                         "description": description,
                         "url": url,
-                        "internal_id_key": id,
-                        "stix_id_key": stix_id_key,
-                        "created": created,
-                        "modified": modified,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "update": update,
                     }
                 },
             )
@@ -179,33 +199,127 @@ class ExternalReference:
             )
 
     """
-        Create a External Reference object only if it not exists, update it on request
+        Upload a file in this External-Reference
 
-        :param name: the name of the External Reference
-        :return External Reference object
+        :param id: the Stix-Domain-Object id
+        :param file_name
+        :param data
+        :return void
     """
 
-    def create(self, **kwargs):
-        source_name = kwargs.get("source_name", None)
-        url = kwargs.get("url", None)
-        external_id = kwargs.get("external_id", None)
-        description = kwargs.get("description", None)
+    def add_file(self, **kwargs):
         id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
-        created = kwargs.get("created", None)
-        modified = kwargs.get("modified", None)
-
-        external_reference_result = self.read(filters=[{"key": "url", "values": [url]}])
-        if external_reference_result is not None:
-            return external_reference_result
-        else:
-            return self.create_raw(
-                source_name=source_name,
-                url=url,
-                external_id=external_id,
-                description=description,
-                id=id,
-                stix_id_key=stix_id_key,
-                created=created,
-                modified=modified,
+        file_name = kwargs.get("file_name", None)
+        data = kwargs.get("data", None)
+        mime_type = kwargs.get("mime_type", "text/plain")
+        if id is not None and file_name is not None:
+            final_file_name = os.path.basename(file_name)
+            query = """
+                mutation ExternalReferenceEdit($id: ID!, $file: Upload!) {
+                    externalReferenceEdit(id: $id) {
+                        importPush(file: $file) {
+                            id
+                            name
+                        }
+                    }
+                }
+             """
+            if data is None:
+                data = open(file_name, "rb")
+                if file_name.endswith(".json"):
+                    mime_type = "application/json"
+                else:
+                    mime_type = magic.from_file(file_name, mime=True)
+            self.opencti.log(
+                "info",
+                "Uploading a file {"
+                + final_file_name
+                + "} in Stix-Domain-Object {"
+                + id
+                + "}.",
             )
+            return self.opencti.query(
+                query,
+                {"id": id, "file": (self.file(final_file_name, data, mime_type))},
+            )
+        else:
+            self.opencti.log(
+                "error",
+                "[opencti_stix_domain_object] Missing parameters: id or file_name",
+            )
+            return None
+
+    """
+        Update a External Reference object field
+
+        :param id: the External Reference id
+        :param input: the input of the field
+        :return The updated External Reference object
+    """
+
+    def update_field(self, **kwargs):
+        id = kwargs.get("id", None)
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            self.opencti.log("info", "Updating External-Reference {" + id + "}.")
+            query = """
+                    mutation ExternalReferenceEdit($id: ID!, $input: [EditInput]!) {
+                        externalReferenceEdit(id: $id) {
+                            fieldPatch(input: $input) {
+                                id
+                            }
+                        }
+                    }
+                """
+            result = self.opencti.query(query, {"id": id, "input": input})
+            return self.opencti.process_multiple_fields(
+                result["data"]["externalReferenceEdit"]["fieldPatch"]
+            )
+        else:
+            self.opencti.log(
+                "error",
+                "[opencti_external_reference] Missing parameters: id and key and value",
+            )
+            return None
+
+    def delete(self, id):
+        self.opencti.log("info", "Deleting External-Reference " + id + "...")
+        query = """
+             mutation ExternalReferenceEdit($id: ID!) {
+                 externalReferenceEdit(id: $id) {
+                     delete
+                 }
+             }
+         """
+        self.opencti.query(query, {"id": id})
+
+    def list_files(self, **kwargs):
+        id = kwargs.get("id", None)
+        self.opencti.log(
+            "info",
+            "Listing files of External-Reference { " + id + " }",
+        )
+        query = """
+            query externalReference($id: String!) {
+                externalReference(id: $id) {
+                    importFiles {
+                        edges {
+                            node {
+                                id
+                                name
+                                size
+                                metaData {
+                                    mimetype
+                                    version
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        """
+        result = self.opencti.query(query, {"id": id})
+        entity = self.opencti.process_multiple_fields(
+            result["data"]["externalReference"]
+        )
+        return entity["importFiles"]
