@@ -8,12 +8,13 @@ class MarkingDefinition:
         self.opencti = opencti
         self.properties = """
             id
-            stix_id_key
+            standard_id
             entity_type
+            parent_types
             definition_type
             definition
-            level
-            color
+            x_opencti_order
+            x_opencti_color
             created
             modified
             created_at
@@ -35,6 +36,7 @@ class MarkingDefinition:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
@@ -51,7 +53,7 @@ class MarkingDefinition:
                     edges {
                         node {
                             """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -129,15 +131,16 @@ class MarkingDefinition:
         :return Marking-Definition object
     """
 
-    def create_raw(self, **kwargs):
-        definition_type = kwargs.get("definition_type", None)
-        definition = kwargs.get("definition", None)
-        level = kwargs.get("level", 0)
-        color = kwargs.get("color", None)
-        id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
+    def create(self, **kwargs):
+        stix_id = kwargs.get("stix_id", None)
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
+        definition_type = kwargs.get("definition_type", None)
+        definition = kwargs.get("definition", None)
+        x_opencti_order = kwargs.get("x_opencti_order", 0)
+        x_opencti_color = kwargs.get("x_opencti_color", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        update = kwargs.get("update", False)
 
         if definition is not None and definition_type is not None:
             query = (
@@ -157,12 +160,13 @@ class MarkingDefinition:
                     "input": {
                         "definition_type": definition_type,
                         "definition": definition,
-                        "internal_id_key": id,
-                        "level": level,
-                        "color": color,
-                        "stix_id_key": stix_id_key,
+                        "x_opencti_order": x_opencti_order,
+                        "x_opencti_color": x_opencti_color,
+                        "stix_id": stix_id,
                         "created": created,
                         "modified": modified,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "update": update,
                     }
                 },
             )
@@ -176,43 +180,107 @@ class MarkingDefinition:
             )
 
     """
-        Create a Marking-Definition object only if it not exists, update it on request
+        Update a Marking definition object field
 
-        :param definition_type: the definition_type
-        :param definition: the definition
-        :return Marking-Definition object
+        :param id: the Marking definition id
+        :param input: the input of the field
+        :return The updated Marking definition object
     """
 
-    def create(self, **kwargs):
-        definition_type = kwargs.get("definition_type", None)
-        definition = kwargs.get("definition", None)
-        level = kwargs.get("level", 0)
-        color = kwargs.get("color", None)
+    def update_field(self, **kwargs):
         id = kwargs.get("id", None)
-        stix_id_key = kwargs.get("stix_id_key", None)
-        created = kwargs.get("created", None)
-        modified = kwargs.get("modified", None)
-
-        object_result = None
-        if stix_id_key is not None:
-            object_result = self.read(id=stix_id_key)
-        if object_result is None:
-            object_result = self.read(
-                filters=[
-                    {"key": "definition_type", "values": [definition_type]},
-                    {"key": "definition", "values": [definition]},
-                ]
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            self.opencti.log("info", "Updating Marking Definition {" + id + "}")
+            query = """
+                    mutation MarkingDefinitionEdit($id: ID!, $input: [EditInput]!) {
+                        markingDefinitionEdit(id: $id) {
+                            fieldPatch(input: $input) {
+                                id
+                                standard_id
+                                entity_type
+                            }
+                        }
+                    }
+                """
+            result = self.opencti.query(
+                query,
+                {
+                    "id": id,
+                    "input": input,
+                },
+            )
+            return self.opencti.process_multiple_fields(
+                result["data"]["markingDefinitionEdit"]["fieldPatch"]
             )
-        if object_result is not None:
-            return object_result
         else:
-            return self.create_raw(
+            self.opencti.log(
+                "error",
+                "[opencti_marking_definition] Missing parameters: id and key and value",
+            )
+            return None
+
+    """
+        Import an Marking Definition object from a STIX2 object
+
+        :param stixObject: the MarkingDefinition
+        :return MarkingDefinition object
+    """
+
+    def import_from_stix2(self, **kwargs):
+        stix_object = kwargs.get("stixObject", None)
+        if stix_object is not None:
+            definition_type = stix_object["definition_type"]
+            definition = stix_object["definition"][stix_object["definition_type"]]
+            if stix_object["definition_type"] == "tlp":
+                definition_type = definition_type.upper()
+                definition = (
+                    definition_type + ":" + stix_object["definition"]["tlp"].upper()
+                )
+
+            # TODO: Compatibility with OpenCTI 3.X to be REMOVED
+            if "x_opencti_order" not in stix_object:
+                stix_object["x_opencti_order"] = (
+                    stix_object["x_opencti_level"]
+                    if "x_opencti_level" in stix_object
+                    else 0
+                )
+
+            return self.opencti.marking_definition.create(
+                stix_id=stix_object["id"],
+                created=stix_object["created"] if "created" in stix_object else None,
+                modified=stix_object["modified"] if "modified" in stix_object else None,
                 definition_type=definition_type,
                 definition=definition,
-                level=level,
-                color=color,
-                id=id,
-                stix_id_key=stix_id_key,
-                created=created,
-                modified=modified,
+                x_opencti_order=stix_object["x_opencti_order"]
+                if "x_opencti_order" in stix_object
+                else 0,
+                x_opencti_color=stix_object["x_opencti_color"]
+                if "x_opencti_color" in stix_object
+                else None,
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
             )
+        else:
+            self.opencti.log(
+                "error", "[opencti_marking_definition] Missing parameters: stixObject"
+            )
+
+    def delete(self, **kwargs):
+        id = kwargs.get("id", None)
+        if id is not None:
+            self.opencti.log("info", "Deleting Marking-Definition {" + id + "}.")
+            query = """
+                 mutation MarkingDefinitionEdit($id: ID!) {
+                     markingDefinitionEdit(id: $id) {
+                         delete
+                     }
+                 }
+             """
+            self.opencti.query(query, {"id": id})
+        else:
+            self.opencti.log(
+                "error", "[opencti_marking_definition] Missing parameters: id"
+            )
+            return None
