@@ -21,6 +21,7 @@ class Note:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -84,6 +85,19 @@ class Note:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -131,6 +145,9 @@ class Note:
                         ... on Sector {
                             name
                         }
+                        ... on System {
+                            name
+                        }                        
                         ... on Indicator {
                             name
                         }
@@ -177,6 +194,19 @@ class Note:
                     }
                 }
             }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
     """
@@ -363,6 +393,7 @@ class Note:
         content = kwargs.get("content", None)
         authors = kwargs.get("authors", None)
         update = kwargs.get("update", False)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
 
         if content is not None:
             self.opencti.log("info", "Creating Note {" + content + "}.")
@@ -393,6 +424,7 @@ class Note:
                         "attribute_abstract": abstract,
                         "content": content,
                         "authors": authors,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
                 },
@@ -543,6 +575,9 @@ class Note:
                 content=self.opencti.stix2.convert_markdown(stix_object["content"])
                 if "content" in stix_object
                 else "",
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
                 authors=stix_object["authors"] if "authors" in stix_object else None,
                 update=update,
             )
