@@ -21,6 +21,7 @@ class ObservedData:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -84,6 +85,19 @@ class ObservedData:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -131,6 +145,9 @@ class ObservedData:
                         ... on Sector {
                             name
                         }
+                        ... on System {
+                            name
+                        }
                         ... on Indicator {
                             name
                         }
@@ -176,6 +193,19 @@ class ObservedData:
                     }
                 }
             }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
     """
@@ -339,9 +369,15 @@ class ObservedData:
         first_observed = kwargs.get("first_observed", None)
         last_observed = kwargs.get("last_observed", None)
         number_observed = kwargs.get("number_observed", None)
+        objects = kwargs.get("objects", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
-        if first_observed is not None and last_observed is not None:
+        if (
+            first_observed is not None
+            and last_observed is not None
+            and objects is not None
+        ):
             self.opencti.log("info", "Creating ObservedData.")
             query = """
                 mutation ObservedDataAdd($input: ObservedDataAddInput) {
@@ -370,6 +406,8 @@ class ObservedData:
                         "first_observed": first_observed,
                         "last_observed": last_observed,
                         "number_observed": number_observed,
+                        "objects": objects,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
                 },
@@ -380,7 +418,7 @@ class ObservedData:
         else:
             self.opencti.log(
                 "error",
-                "[opencti_observedData] Missing parameters: first_observed or last_observed",
+                "[opencti_observedData] Missing parameters: first_observed, last_observed or objects",
             )
 
     """
@@ -494,6 +532,29 @@ class ObservedData:
         stix_object = kwargs.get("stixObject", None)
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
+
+        if "objects" in stix_object:
+            stix_observable_results = []
+            for key, observable_item in stix_object["objects"].items():
+                stix_observable_results.append(
+                    self.opencti.stix_cyber_observable.create(
+                        observableData=observable_item,
+                        createdBy=extras["created_by_id"]
+                        if "created_by_id" in extras
+                        else None,
+                        objectMarking=extras["object_marking_ids"]
+                        if "object_marking_ids" in extras
+                        else None,
+                        objectLabel=extras["object_label_ids"]
+                        if "object_label_ids" in extras
+                        else [],
+                    )
+                )
+                refs = []
+                for item in stix_observable_results:
+                    refs.append(item["standard_id"])
+                stix_object["object_refs"] = refs
+
         if stix_object is not None:
             observed_data_result = self.create(
                 stix_id=stix_object["id"],
@@ -525,33 +586,15 @@ class ObservedData:
                 number_observed=stix_object["number_observed"]
                 if "number_observed" in stix_object
                 else None,
+                objects=stix_object["object_refs"]
+                if "object_refs" in stix_object
+                else None,
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
                 update=update,
             )
-            if "objects" in stix_object:
-                for key, observable_item in stix_object["objects"].items():
-                    stix_observable_result = self.opencti.stix_cyber_observable.create(
-                        observableData=observable_item,
-                        createdBy=extras["created_by_id"]
-                        if "created_by_id" in extras
-                        else None,
-                        objectMarking=extras["object_marking_ids"]
-                        if "object_marking_ids" in extras
-                        else None,
-                        objectLabel=extras["object_label_ids"]
-                        if "object_label_ids" in extras
-                        else [],
-                    )
-                    if stix_observable_result is not None:
-                        self.add_stix_object_or_stix_relationship(
-                            id=observed_data_result["id"],
-                            stixObjectOrStixRelationshipId=stix_observable_result["id"],
-                        )
-                        self.opencti.stix2.mapping_cache[
-                            stix_observable_result["id"]
-                        ] = {
-                            "id": stix_observable_result["id"],
-                            "type": stix_observable_result["entity_type"],
-                        }
+
             return observed_data_result
         else:
             self.opencti.log(
