@@ -1,6 +1,9 @@
 # coding: utf-8
 
 import json
+import os
+
+import magic
 
 
 class StixCyberObservable:
@@ -22,6 +25,7 @@ class StixCyberObservable:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -85,6 +89,19 @@ class StixCyberObservable:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -277,12 +294,28 @@ class StixCyberObservable:
             ... on XOpenCTICryptocurrencyWallet {
                 value
             }
+            ... on XOpenCTIHostname {
+                value
+            }
             ... on XOpenCTIText {
                 value
             }
             ... on XOpenCTIUserAgent {
                 value
             }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
     """
@@ -317,11 +350,11 @@ class StixCyberObservable:
         )
         query = (
             """
-            query StixCyberObservables($types: [String], $filters: [StixCyberObservablesFiltering], $search: String, $first: Int, $after: ID, $orderBy: StixCyberObservablesOrdering, $orderMode: OrderingMode) {
-                stixCyberObservables(types: $types, filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
-                    edges {
-                        node {
-                            """
+                query StixCyberObservables($types: [String], $filters: [StixCyberObservablesFiltering], $search: String, $first: Int, $after: ID, $orderBy: StixCyberObservablesOrdering, $orderMode: OrderingMode) {
+                    stixCyberObservables(types: $types, filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
+                        edges {
+                            node {
+                                """
             + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
@@ -395,9 +428,9 @@ class StixCyberObservable:
             self.opencti.log("info", "Reading StixCyberObservable {" + id + "}.")
             query = (
                 """
-                query StixCyberObservable($id: String!) {
-                    stixCyberObservable(id: $id) {
-                        """
+                    query StixCyberObservable($id: String!) {
+                        stixCyberObservable(id: $id) {
+                            """
                 + (
                     custom_attributes
                     if custom_attributes is not None
@@ -425,6 +458,57 @@ class StixCyberObservable:
             )
             return None
 
+    """
+        Upload a file in this Observable
+
+        :param id: the Stix-Cyber-Observable id
+        :param file_name
+        :param data
+        :return void
+    """
+
+    def add_file(self, **kwargs):
+        id = kwargs.get("id", None)
+        file_name = kwargs.get("file_name", None)
+        data = kwargs.get("data", None)
+        mime_type = kwargs.get("mime_type", "text/plain")
+        if id is not None and file_name is not None:
+            final_file_name = os.path.basename(file_name)
+            query = """
+                    mutation StixCyberObservableEdit($id: ID!, $file: Upload!) {
+                        stixCyberObservableEdit(id: $id) {
+                            importPush(file: $file) {
+                                id
+                                name
+                            }
+                        }
+                    }
+                 """
+            if data is None:
+                data = open(file_name, "rb")
+                if file_name.endswith(".json"):
+                    mime_type = "application/json"
+                else:
+                    mime_type = magic.from_file(file_name, mime=True)
+            self.opencti.log(
+                "info",
+                "Uploading a file {"
+                + final_file_name
+                + "} in Stix-Cyber-Observable {"
+                + id
+                + "}.",
+            )
+            return self.opencti.query(
+                query,
+                {"id": id, "file": (self.file(final_file_name, data, mime_type))},
+            )
+        else:
+            self.opencti.log(
+                "error",
+                "[opencti_stix_cyber_observable Missing parameters: id or file_name",
+            )
+            return None
+
     """
         Create a Stix-Observable object
 
@@ -1064,32 +1148,187 @@ class StixCyberObservable:
         else:
             self.opencti.log("error", "Missing parameters: type")
 
+    """
+        Upload an artifact
+
+        :param file_path: the file path
+        :return Stix-Observable object
+    """
+
+    def upload_artifact(self, **kwargs):
+        file_name = kwargs.get("file_name", None)
+        data = kwargs.get("data", None)
+        mime_type = kwargs.get("mime_type", "text/plain")
+        x_opencti_description = kwargs.get("x_opencti_description", False)
+        created_by = kwargs.get("createdBy", None)
+        object_marking = kwargs.get("objectMarking", None)
+        object_label = kwargs.get("objectLabel", None)
+        create_indicator = kwargs.get("createIndicator", False)
+
+        if file_name is not None and mime_type is not None:
+            final_file_name = os.path.basename(file_name)
+            self.opencti.log(
+                "info",
+                "Creating Stix-Cyber-Observable {artifact}} with indicator at "
+                + str(create_indicator)
+                + ".",
+            )
+            query = """
+                mutation ArtifactImport($file: Upload!, $x_opencti_description: String, $createdBy: String, $objectMarking: [String], $objectLabel: [String]) {
+                    artifactImport(file: $file, x_opencti_description: $x_opencti_description, createdBy: $createdBy, objectMarking: $objectMarking, objectLabel: $objectLabel) {
+                        id
+                        standard_id
+                        entity_type
+                        parent_types
+                        spec_version
+                        created_at
+                        updated_at
+                        createdBy {
+                            ... on Identity {
+                                id
+                                standard_id
+                                entity_type
+                                parent_types
+                                spec_version
+                                name
+                                description
+                                roles
+                                contact_information
+                                x_opencti_aliases
+                                created
+                                modified
+                                objectLabel {
+                                    edges {
+                                        node {
+                                            id
+                                            value
+                                            color
+                                        }
+                                    }
+                                }
+                            }
+                            ... on Organization {
+                                x_opencti_organization_type
+                                x_opencti_reliability
+                            }
+                            ... on Individual {
+                                x_opencti_firstname
+                                x_opencti_lastname
+                            }
+                        }
+                        objectMarking {
+                            edges {
+                                node {
+                                    id
+                                    standard_id
+                                    entity_type
+                                    definition_type
+                                    definition
+                                    created
+                                    modified
+                                    x_opencti_order
+                                    x_opencti_color
+                                }
+                            }
+                        }
+                        objectLabel {
+                            edges {
+                                node {
+                                    id
+                                    value
+                                    color
+                                }
+                            }
+                        }
+                        externalReferences {
+                            edges {
+                                node {
+                                    id
+                                    standard_id
+                                    entity_type
+                                    source_name
+                                    description
+                                    url
+                                    hash
+                                    external_id
+                                    created
+                                    modified
+                                }
+                            }
+                        }
+                        observable_value
+                        x_opencti_description
+                        x_opencti_score
+                        indicators {
+                            edges {
+                                node {
+                                    id
+                                    pattern
+                                    pattern_type
+                                }
+                            }
+                        }
+                        mime_type
+                        payload_bin
+                        url
+                        encryption_algorithm
+                        decryption_key
+                        hashes {
+                            algorithm
+                            hash
+                        }
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                }
+                            }
+                        }
+                    }
+                }
+            """
+            if data is None:
+                data = open(file_name, "rb")
+                if file_name.endswith(".json"):
+                    mime_type = "application/json"
+                else:
+                    mime_type = magic.from_file(file_name, mime=True)
+
+            result = self.opencti.query(
+                query,
+                {
+                    "file": (self.file(final_file_name, data, mime_type)),
+                    "x_opencti_description": x_opencti_description,
+                    "createdBy": created_by,
+                    "objectMarking": object_marking,
+                    "objectLabel": object_label,
+                },
+            )
+            return self.opencti.process_multiple_fields(
+                result["data"]["artifactImport"]
+            )
+        else:
+            self.opencti.log("error", "Missing parameters: type")
+
     """
         Update a Stix-Observable object field
 
         :param id: the Stix-Observable id
-        :param key: the key of the field
-        :param value: the value of the field
+        :param input: the input of the field
         :return The updated Stix-Observable object
     """
 
     def update_field(self, **kwargs):
         id = kwargs.get("id", None)
-        key = kwargs.get("key", None)
-        value = kwargs.get("value", None)
-        operation = kwargs.get("operation", "replace")
-        if isinstance(value, list):
-            value = [str(v) for v in value]
-        else:
-            value = str(value)
-        if id is not None and key is not None and value is not None:
-            self.opencti.log(
-                "info", "Updating Stix-Observable {" + id + "} field {" + key + "}."
-            )
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            self.opencti.log("info", "Updating Stix-Observable {" + id + "}.")
             query = """
-                mutation StixCyberObservableEdit($id: ID!, $input: EditInput!, $operation: EditOperation) {
+                mutation StixCyberObservableEdit($id: ID!, $input: [EditInput]!) {
                     stixCyberObservableEdit(id: $id) {
-                        fieldPatch(input: $input, operation: $operation) {
+                        fieldPatch(input: $input) {
                             id
                             standard_id
                             entity_type
@@ -1101,8 +1340,7 @@ class StixCyberObservable:
                 query,
                 {
                     "id": id,
-                    "input": {"key": key, "value": value},
-                    "operation": operation,
+                    "input": input,
                 },
             )
             return self.opencti.process_multiple_fields(
@@ -1111,7 +1349,7 @@ class StixCyberObservable:
         else:
             self.opencti.log(
                 "error",
-                "[opencti_stix_cyber_observable_update_field] Missing parameters: id and key and value",
+                "[opencti_stix_cyber_observable_update_field] Missing parameters: id and input",
             )
             return None
 
@@ -1569,3 +1807,31 @@ class StixCyberObservable:
                 "listFilters": list_filters,
             },
         )
+
+    def ask_for_enrichment(self, **kwargs) -> str:
+        id = kwargs.get("id", None)
+        connector_id = kwargs.get("connector_id", None)
+
+        if id is None or connector_id is None:
+            self.opencti.log("error", "Missing parameters: id and connector_id")
+            return ""
+
+        query = """
+            mutation StixCoreObjectEnrichmentLinesMutation($id: ID!, $connectorId: ID!) {
+                stixCoreObjectEdit(id: $id) {
+                    askEnrichment(connectorId: $connectorId) {
+                        id
+                    }
+                }
+            }
+            """
+
+        result = self.opencti.query(
+            query,
+            {
+                "id": id,
+                "connectorId": connector_id,
+            },
+        )
+        # return work_id
+        return result["data"]["stixCoreObjectEdit"]["askEnrichment"]["id"]
