@@ -21,6 +21,7 @@ class Tool:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -84,6 +85,19 @@ class Tool:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -110,6 +124,19 @@ class Tool:
                     }
                 }
             }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
     """
@@ -129,6 +156,7 @@ class Tool:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
@@ -144,7 +172,7 @@ class Tool:
                     edges {
                         node {
                             """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -263,6 +291,7 @@ class Tool:
         tool_types = kwargs.get("tool_types", None)
         tool_version = kwargs.get("tool_version", None)
         kill_chain_phases = kwargs.get("killChainPhases", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None:
@@ -297,6 +326,7 @@ class Tool:
                         "tool_types": tool_types,
                         "tool_version": tool_version,
                         "killChainPhases": kill_chain_phases,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
                 },
@@ -356,6 +386,9 @@ class Tool:
                 killChainPhases=extras["kill_chain_phases_ids"]
                 if "kill_chain_phases_ids" in extras
                 else None,
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
                 update=update,
             )
         else:
