@@ -5,15 +5,17 @@ import datetime
 import json
 import os
 import uuid
-from typing import List
+from typing import List, Any, Dict, Optional, Union
 
 import datefinder
 import dateutil.parser
 import pytz
 
+from pycti.entities.opencti_identity import Identity
 from pycti.utils.constants import IdentityTypes, LocationTypes, StixCyberObservableTypes
 from pycti.utils.opencti_stix2_splitter import OpenCTIStix2Splitter
 from pycti.utils.opencti_stix2_update import OpenCTIStix2Update
+from pycti.utils.opencti_stix2_utils import OBSERVABLES_VALUE_INT
 
 datefinder.ValueError = ValueError, OverflowError
 utc = pytz.UTC
@@ -35,13 +37,13 @@ class OpenCTIStix2:
 
     ######### UTILS
     # region utils
-    def unknown_type(self, stix_object):
+    def unknown_type(self, stix_object: Dict) -> None:
         self.opencti.log(
             "error",
             'Unknown object type "' + stix_object["type"] + '", doing nothing...',
         )
 
-    def convert_markdown(self, text) -> str:
+    def convert_markdown(self, text: str) -> str:
         """converts input text to markdown style code annotation
 
         :param text: input text
@@ -52,31 +54,33 @@ class OpenCTIStix2:
 
         return text.replace("<code>", "`").replace("</code>", "`")
 
-    def format_date(self, date):
+    def format_date(self, date: Any = None) -> str:
         """converts multiple input date formats to OpenCTI style dates
 
         :param date: input date
-        :type date:
+        :type date: Any [datetime, date, str or none]
         :return: OpenCTI style date
-        :rtype: datetime
+        :rtype: string
         """
-
-        if isinstance(date, datetime.date):
-            return date.isoformat(timespec="milliseconds").replace("+00:00", "Z")
-        if date is not None:
-            return (
-                dateutil.parser.parse(date)
-                .isoformat(timespec="milliseconds")
-                .replace("+00:00", "Z")
-            )
+        if isinstance(date, datetime.datetime):
+            date_value = date
+        elif isinstance(date, datetime.date):
+            date_value = datetime.datetime.combine(date, datetime.datetime.min.time())
+        elif isinstance(date, str):
+            try:
+                date_value = dateutil.parser.parse(date)
+            except (dateutil.parser.ParserError, TypeError, OverflowError) as e:
+                raise ValueError(f"{e}: {date} does not contain a valid date string")
         else:
-            return (
-                datetime.datetime.utcnow()
-                .isoformat(timespec="milliseconds")
-                .replace("+00:00", "Z")
-            )
+            date_value = datetime.datetime.utcnow()
 
-    def filter_objects(self, uuids: list, objects: list) -> list:
+        if not date_value.tzinfo:
+            self.opencti.log("No timezone found. Setting to UTC", "info")
+            date_value = date_value.replace(tzinfo=datetime.timezone.utc)
+
+        return date_value.isoformat(timespec="milliseconds").replace("+00:00", "Z")
+
+    def filter_objects(self, uuids: List, objects: List) -> List:
         """filters objects based on UUIDs
 
         :param uuids: list of UUIDs
@@ -94,7 +98,7 @@ class OpenCTIStix2:
                     result.append(item)
         return result
 
-    def pick_aliases(self, stix_object) -> list:
+    def pick_aliases(self, stix_object: Dict) -> Optional[List]:
         """check stix2 object for multiple aliases and return a list
 
         :param stix_object: valid stix2 object
@@ -115,7 +119,7 @@ class OpenCTIStix2:
         return None
 
     def check_max_marking_definition(
-        self, max_marking_definition_entity: str, entity_marking_definitions: list
+        self, max_marking_definition_entity: Dict, entity_marking_definitions: List
     ) -> bool:
         """checks if a list of marking definitions conforms with a given max level
 
@@ -151,7 +155,9 @@ class OpenCTIStix2:
                 return True
         return False
 
-    def import_bundle_from_file(self, file_path: str, update=False, types=None) -> List:
+    def import_bundle_from_file(
+        self, file_path: str, update: bool = False, types: List = None
+    ) -> Optional[List]:
         """import a stix2 bundle from a file
 
         :param file_path: valid path to the file
@@ -171,7 +177,11 @@ class OpenCTIStix2:
         return self.import_bundle(data, update, types)
 
     def import_bundle_from_json(
-        self, json_data, update=False, types=None, retry_number=None
+        self,
+        json_data: Union[str, bytes],
+        update: bool = False,
+        types: List = None,
+        retry_number: int = None,
     ) -> List:
         """import a stix2 bundle from JSON data
 
@@ -192,7 +202,7 @@ class OpenCTIStix2:
             types,
         )
 
-    def resolve_author(self, title):
+    def resolve_author(self, title: str) -> Optional[Identity]:
         if "fireeye" in title.lower() or "mandiant" in title.lower():
             return self.get_author("FireEye")
         if "eset" in title.lower():
@@ -233,7 +243,7 @@ class OpenCTIStix2:
             return self.get_author("The MITRE Corporation")
         return None
 
-    def get_author(self, name):
+    def get_author(self, name: str) -> Identity:
         if name in self.mapping_cache:
             return self.mapping_cache[name]
         else:
@@ -245,7 +255,9 @@ class OpenCTIStix2:
             self.mapping_cache[name] = author
             return author
 
-    def extract_embedded_relationships(self, stix_object, types=None) -> dict:
+    def extract_embedded_relationships(
+        self, stix_object: Dict, types: List = None
+    ) -> Dict:
         """extracts embedded relationship objects from a stix2 entity
 
         :param stix_object: valid stix2 object
@@ -354,6 +366,14 @@ class OpenCTIStix2:
                         if "description" in external_reference
                         else None,
                     )["id"]
+                if "x_opencti_files" in external_reference:
+                    for file in external_reference["x_opencti_files"]:
+                        self.opencti.external_reference.add_file(
+                            id=external_reference_id,
+                            file_name=file["name"],
+                            data=base64.b64decode(file["data"]),
+                            mime_type=file["mime_type"],
+                        )
                 self.mapping_cache[url] = {"id": external_reference_id}
                 external_references_ids.append(external_reference_id)
                 if (
@@ -444,7 +464,6 @@ class OpenCTIStix2:
                         else "",
                         report_types="threat-report",
                         published=published,
-                        x_opencti_report_status=2,
                         update=True,
                     )
                     reports[external_reference_id] = report
@@ -462,7 +481,9 @@ class OpenCTIStix2:
     # endregion
 
     # region import
-    def import_object(self, stix_object, update=False, types=None) -> list:
+    def import_object(
+        self, stix_object: Dict, update: bool = False, types: List = None
+    ) -> Optional[List]:
         """import a stix2 object
 
         :param stix_object: valid stix2 object
@@ -539,7 +560,7 @@ class OpenCTIStix2:
         )
 
         if stix_object_results is None:
-            return stix_object_results
+            return None
 
         if not isinstance(stix_object_results, list):
             stix_object_results = [stix_object_results]
@@ -629,7 +650,9 @@ class OpenCTIStix2:
 
         return stix_object_results
 
-    def import_observable(self, stix_object, update=False, types=None):
+    def import_observable(
+        self, stix_object: Dict, update: bool = False, types: List = None
+    ) -> None:
         # Extract
         embedded_relationships = self.extract_embedded_relationships(stix_object, types)
         created_by_id = embedded_relationships["created_by"]
@@ -654,7 +677,9 @@ class OpenCTIStix2:
             stix_observable_result = self.opencti.stix_cyber_observable.create(
                 simple_observable_id=stix_object["id"],
                 simple_observable_key=stix_object["key"],
-                simple_observable_value=stix_object["value"],
+                simple_observable_value=stix_object["value"]
+                if stix_object["key"] not in OBSERVABLES_VALUE_INT
+                else int(stix_object["value"]),
                 simple_observable_description=stix_object["description"]
                 if "description" in stix_object
                 else None,
@@ -696,6 +721,15 @@ class OpenCTIStix2:
                 update=update,
             )
         if stix_observable_result is not None:
+            # Add files
+            if "x_opencti_files" in stix_object:
+                for file in stix_object["x_opencti_files"]:
+                    self.opencti.stix_cyber_observable.add_file(
+                        id=stix_observable_result["id"],
+                        file_name=file["name"],
+                        data=base64.b64decode(file["data"]),
+                        mime_type=file["mime_type"],
+                    )
             if "id" in stix_object:
                 self.mapping_cache[stix_object["id"]] = {
                     "id": stix_observable_result["id"],
@@ -708,7 +742,9 @@ class OpenCTIStix2:
         else:
             return None
 
-    def import_relationship(self, stix_relation, update=False, types=None):
+    def import_relationship(
+        self, stix_relation: Dict, update: bool = False, types: List = None
+    ) -> None:
         # Extract
         embedded_relationships = self.extract_embedded_relationships(
             stix_relation, types
@@ -792,7 +828,14 @@ class OpenCTIStix2:
                     stixObjectOrStixRelationshipId=stix_relation["target_ref"],
                 )
 
-    def import_sighting(self, stix_sighting, from_id, to_id, update=False, types=None):
+    def import_sighting(
+        self,
+        stix_sighting: Dict,
+        from_id: str,
+        to_id: str,
+        update: bool = False,
+        types: List = None,
+    ) -> None:
         # Extract
         embedded_relationships = self.extract_embedded_relationships(
             stix_sighting, types
@@ -900,7 +943,7 @@ class OpenCTIStix2:
     # endregion
 
     # region export
-    def generate_export(self, entity):
+    def generate_export(self, entity: Dict) -> Dict:
         # Handle model deviation
         # Identities
         if IdentityTypes.has_value(entity["entity_type"]):
@@ -967,6 +1010,27 @@ class OpenCTIStix2:
                     external_reference["external_id"] = entity_external_reference[
                         "external_id"
                     ]
+                if (
+                    "importFiles" in entity_external_reference
+                    and len(entity_external_reference["importFiles"]) > 0
+                ):
+                    external_reference["x_opencti_files"] = []
+                    for file in entity_external_reference["importFiles"]:
+                        url = (
+                            self.opencti.api_url.replace("graphql", "storage/get/")
+                            + file["id"]
+                        )
+                        data = self.opencti.fetch_opencti_file(
+                            url, binary=True, serialize=True
+                        )
+                        external_reference["x_opencti_files"].append(
+                            {
+                                "name": file["name"],
+                                "data": data,
+                                "mime_type": file["metaData"]["mimetype"],
+                                "version": file["metaData"]["version"],
+                            }
+                        )
                 entity["external_references"].append(external_reference)
         if "externalReferences" in entity:
             del entity["externalReferences"]
@@ -996,15 +1060,15 @@ class OpenCTIStix2:
 
     def prepare_export(
         self,
-        entity,
-        mode="simple",
-        max_marking_definition_entity=None,
-        no_custom_attributes=False,
-    ):
+        entity: Dict,
+        mode: str = "simple",
+        max_marking_definition_entity: Dict = None,
+        no_custom_attributes: bool = False,
+    ) -> List:
         if (
             self.check_max_marking_definition(
                 max_marking_definition_entity,
-                entity["objectMarking"] if "objectMarking" in entity else None,
+                entity["objectMarking"] if "objectMarking" in entity else [],
             )
             is False
         ):
@@ -1109,7 +1173,7 @@ class OpenCTIStix2:
             del entity["attribute_count"]
             entity["sighting_of_ref"] = entity["from"]["standard_id"]
             objects_to_get.append(entity["from"]["standard_id"])
-            entity["where_sighted_refs"] = entity["to"]["standard_id"]
+            entity["where_sighted_refs"] = [entity["to"]["standard_id"]]
             objects_to_get.append(entity["to"]["standard_id"])
             del entity["from"]
             del entity["to"]
@@ -1139,6 +1203,31 @@ class OpenCTIStix2:
         if "attribute_date" in entity:
             entity["date"] = entity["attribute_date"]
             del entity["attribute_date"]
+        # Artifact
+        if entity["type"] == "artifact" and "importFiles" in entity:
+            first_file = entity["importFiles"][0]["id"]
+            url = self.opencti.api_url.replace("graphql", "storage/get/") + first_file
+            file = self.opencti.fetch_opencti_file(url, binary=True, serialize=True)
+            if file:
+                entity["payload_bin"] = file
+        # Files
+        if "importFiles" in entity and len(entity["importFiles"]) > 0:
+            entity["x_opencti_files"] = []
+            for file in entity["importFiles"]:
+                url = (
+                    self.opencti.api_url.replace("graphql", "storage/get/") + file["id"]
+                )
+                data = self.opencti.fetch_opencti_file(url, binary=True, serialize=True)
+                entity["x_opencti_files"].append(
+                    {
+                        "name": file["name"],
+                        "data": data,
+                        "mime_type": file["metaData"]["mimetype"],
+                        "version": file["metaData"]["version"],
+                    }
+                )
+            del entity["importFiles"]
+            del entity["importFilesIds"]
 
         result.append(entity)
 
@@ -1333,12 +1422,12 @@ class OpenCTIStix2:
 
     def export_entity(
         self,
-        entity_type,
-        entity_id,
-        mode="simple",
-        max_marking_definition=None,
-        no_custom_attributes=False,
-    ):
+        entity_type: str,
+        entity_id: str,
+        mode: str = "simple",
+        max_marking_definition: Dict = None,
+        no_custom_attributes: bool = False,
+    ) -> Dict:
         max_marking_definition_entity = (
             self.opencti.marking_definition.read(id=max_marking_definition)
             if max_marking_definition is not None
@@ -1374,7 +1463,10 @@ class OpenCTIStix2:
             "Tool": self.opencti.tool.read,
             "Vulnerability": self.opencti.vulnerability.read,
             "Incident": self.opencti.incident.read,
+            "Stix-Cyber-Observable": self.opencti.stix_cyber_observable.read,
         }
+        if StixCyberObservableTypes.has_value(entity_type):
+            entity_type = "Stix-Cyber-Observable"
         do_read = reader.get(
             entity_type, lambda **kwargs: self.unknown_type({"type": entity_type})
         )
@@ -1394,14 +1486,14 @@ class OpenCTIStix2:
 
     def export_list(
         self,
-        entity_type,
-        search=None,
-        filters=None,
-        order_by=None,
-        order_mode=None,
-        max_marking_definition=None,
-        types=None,
-    ):
+        entity_type: str,
+        search: Dict = None,
+        filters: List = None,
+        order_by: str = None,
+        order_mode: str = None,
+        max_marking_definition: Dict = None,
+        types: List = None,
+    ) -> Dict:
         max_marking_definition_entity = (
             self.opencti.marking_definition.read(id=max_marking_definition)
             if max_marking_definition is not None
@@ -1484,7 +1576,9 @@ class OpenCTIStix2:
                     bundle["objects"] = bundle["objects"] + entity_bundle_filtered
         return bundle
 
-    def import_bundle(self, stix_bundle, update=False, types=None) -> List:
+    def import_bundle(
+        self, stix_bundle: Dict, update: bool = False, types: List = None
+    ) -> List:
         # Check if the bundle is correctly formatted
         if "type" not in stix_bundle or stix_bundle["type"] != "bundle":
             raise ValueError("JSON data type is not a STIX2 bundle")
@@ -1505,13 +1599,9 @@ class OpenCTIStix2:
         for bundle in bundles:
             for item in bundle["objects"]:
                 if "x_opencti_event_version" in bundle:
-                    if bundle["x_opencti_event_version"] == "1":
-                        if "x_data_update" in item:
-                            self.stix2_update.process_update_v1(item)
-                            continue
-                    elif bundle["x_opencti_event_version"] == "2":
+                    if bundle["x_opencti_event_version"] == "3":
                         if "x_opencti_patch" in item:
-                            self.stix2_update.process_update_v2(item)
+                            self.stix2_update.process_update(item)
                             continue
                 if item["type"] == "relationship":
                     self.import_relationship(item, update, types)
@@ -1522,11 +1612,7 @@ class OpenCTIStix2:
                         for where_sighted_ref in item["where_sighted_refs"]:
                             to_ids.append(where_sighted_ref)
                     # Import sighting_of_ref
-                    from_id = (
-                        item["x_opencti_sighting_of_ref"]
-                        if "x_opencti_sighting_of_ref" in item
-                        else item["sighting_of_ref"]
-                    )
+                    from_id = item["sighting_of_ref"]
                     if len(to_ids) > 0:
                         for to_id in to_ids:
                             self.import_sighting(item, from_id, to_id, update)
@@ -1538,6 +1624,12 @@ class OpenCTIStix2:
                                     self.import_sighting(
                                         item, observed_data_ref, to_id, update
                                     )
+                elif item["type"] == "label":
+                    self.opencti.label.create(**item)
+                elif item["type"] == "external-reference":
+                    self.opencti.external_reference.create(**item)
+                elif item["type"] == "kill-chain-phase":
+                    self.opencti.kill_chain_phase.create(**item)
                 elif StixCyberObservableTypes.has_value(item["type"]):
                     if types is None or len(types) == 0:
                         self.import_observable(item, update, types)
